
# Classical game engine
from .engine import Engine
from .player import ControllablePlayer
from .collider import Collider
from .physics import PhysicsObject
from .animation import Animations

# Individual learning games


# Other
# Gør det nemere at impotere for læring
# NOTE: Can of course create more namespace
from pyglet.shapes import Rectangle, Circle, Line, Arc, Ellipse, Triangle, Polygon, BezierCurve
